#include "reboot_temperature.h"

int get_temperature(char *path)
{
    char buffer[SIZE];
    FILE *fp;

    if ((fp = fopen(path, "r")) < 0)
        return -1;              /* 文件不存在,则退出. */

    while (fgets(buffer, SIZE, fp) != NULL) ;

    fclose(fp);
    return atoi(buffer) / 1000;
}

int error_log(int l_t, int c_t, char *log_file)
{
    time_t tmpcal_ptr;
    struct tm *tmp_ptr = NULL;
    FILE *fp;
    char buffer[SIZE];
    char temperature[SIZE];

    memset(buffer, 0, SIZE);
    memset(temperature, 0, SIZE);
    time(&tmpcal_ptr);
    tmp_ptr = localtime(&tmpcal_ptr);

    FILE *fd = fopen(log_file, "a+");
    if (fd == NULL)
        return -1;

    // 写入日志文件FD
    fprintf(fd, "%d.%d.%d %d:%d:%d %s %d %s %d\n", (1900 + tmp_ptr->tm_year), (1 + tmp_ptr->tm_mon), tmp_ptr->tm_mday, tmp_ptr->tm_hour, tmp_ptr->tm_min, tmp_ptr->tm_sec, "树莓CPU温度", l_t, "达到可重启温度", c_t);

    // 发送邮件字符串
    sprintf(temperature, "%d.%d.%d %d:%d:%d %s %d %s %d\n", (1900 + tmp_ptr->tm_year), (1 + tmp_ptr->tm_mon), tmp_ptr->tm_mday, tmp_ptr->tm_hour, tmp_ptr->tm_min, tmp_ptr->tm_sec, "树莓CPU温度", l_t, "达到可重启温度", c_t);

    sprintf(buffer, "/root/qqMail/qqMail -l smtp.qq.com -p 25 -f 1605227279 -e dqqpbbbxoazibafd -q NIUYULING -r 1605227279@QQ.COM -n NIUYULING -s \"Raspberrypi Temperature\" -t \"%s\"", temperature);
    //printf("%s\n", buffer);

    fp = popen(buffer, "r");
    pclose(fp);
    fclose(fd);

    return 0;
}

int correct_log(int l_t, int c_t, char *log_file)
{
    time_t tmpcal_ptr;
    struct tm *tmp_ptr = NULL;

    time(&tmpcal_ptr);
    tmp_ptr = localtime(&tmpcal_ptr);
    FILE *fd = fopen(log_file, "a+");
    if (fd == NULL)
        return -1;

    fprintf(fd, "%d.%d.%d %d:%d:%d %s %d %s %d\n", (1900 + tmp_ptr->tm_year), (1 + tmp_ptr->tm_mon), tmp_ptr->tm_mday, tmp_ptr->tm_hour, tmp_ptr->tm_min, tmp_ptr->tm_sec, "树莓派CPU温度", l_t, "未达到重启或者关机温度", c_t);
    fclose(fd);

    return 0;
}

int get_executable_path(char *processdir, char *processname, int len)
{
    char *filename;

    if (readlink("/proc/self/exe", processdir, len) <= 0)
        return -1;
    filename = strrchr(processdir, '/');
    if (filename == NULL)
        return -1;
    ++filename;
    strcpy(processname, filename);
    *filename = '\0';

    return (int)(filename - processdir);
}

int main(int argc, char *argv[], char **env)
{
    char buffer[SIZE];
    char log_file[SIZE];
    char *inifile = "conf/config.ini";
    char path[SIZE] = { 0 };
    char executable_filename[SIZE] = { 0 };
    (void)get_executable_path(path, executable_filename, sizeof(path));
    inifile = strcat(path, inifile);

    memset(buffer, 0, SIZE);
    memset(log_file, 0, SIZE);

    if (daemon(1, 1) == -1) {
        perror("daemon");
        exit(1);
    }

    if (-1 == (nice(getinikeyint("server", "nice", inifile)))) { // 进程优先级
        perror("nice");
    }

    while (1) {
        getinikeystring("server", "thermal_zone", inifile, buffer); // 获取thermal_zone路径
        getinikeystring("server", "log_file", inifile, log_file); // 获取日志文件名

        if (get_temperature(buffer) >= getinikeyint("server", "temperature", inifile)) { // 达到重启或者关机温度
            sync();
            if (getinikeyint("server", "off_power", inifile) == 1) {
                error_log(get_temperature(buffer), getinikeyint("server", "temperature", inifile), log_file);
                return reboot(RB_POWER_OFF); // 关机

            } else {
                error_log(get_temperature(buffer), getinikeyint("server", "temperature", inifile), log_file);
                return reboot(RB_AUTOBOOT); // 重启

            }
        } else {                // 未达到重启或者关机温度
            correct_log(get_temperature(buffer), getinikeyint("server", "temperature", inifile), log_file);
        }

        sleep(getinikeyint("server", "second", inifile));
    }

    return 0;
}
